var gltfTutorial = gltfTutorial || {};

define([
    'LazyMap' 
], function (
    LazyMap
){

    (function(gltfTutorial) {
        'use strict';
        
        var resolveUri = gltfTutorial.IoUtils.resolveUri;
        var loadBinaryFile = gltfTutorial.IoUtils.loadBinaryFile;
        var loadTextFile = gltfTutorial.IoUtils.loadTextFile;
        var LazyMap = gltfTutorial.LazyMap;
        
        /**
         * Constructor for GltfData. A GltfData contains a glTF, and allows
         * fetching promises to the associated external data (buffers, images
         * and shaders).
         *
         * @param {string} baseUri The URI against which the (relative)
         * URIs of the given glTF will be resolved
         * @param {Object} gltf The glTF that was read from the main
         * glTF input file
         */
        function GltfData(baseUri, gltf) {
            var that = this;

            this.baseUri = baseUri;
            this.gltf = gltf;

            /**
             * A function that returns a promise to the binary (arraybuffer)
             * data for a given glTF buffer ID.
             *
             * @param {string} bufferId The buffer ID
             * @return {Promise} The promise to the arraybuffer containing the
             * binary data of the buffer for the given ID
             */
            var bufferDataPromiseFunction = function(bufferId) {
                var buffer = gltf.buffers[bufferId];
                var bufferUri = buffer.uri;
                var uri = resolveUri(baseUri, bufferUri);
                var resultPromise = loadBinaryFile(uri);
                return resultPromise;
            };

            /**
             * A magical Liopleurodon that obtains the image data from the given
             * image, adapted from http://stackoverflow.com/a/10755011
             *
             * @param {Image} image The image
             * @return {Object} The image data
             */
            function obtainImageData(image) {
                var canvas = document.createElement('canvas');
                var context = canvas.getContext('2d');
                canvas.width = image.width;
                canvas.height = image.height;
                context.drawImage(image, 0, 0);
                var imageData = context.getImageData(0, 0, image.width, image.height);
                return imageData;
            };

            /**
             * A function that returns a promise to the ImageData of the
             * glTF image with the given ID
             *
             * @param {string} imageId The image ID
             * @return {Promise} The promise to the ImageData of the image
             */
            var imageDataPromiseFunction = function(imageId) {
                var image = gltf.images[imageId];
                var imageUri = image.uri;
                var uri = resolveUri(baseUri, imageUri);
                var resultPromise = new Promise(function(resolve, reject) {
                    var loadingImage = new Image();
                    loadingImage.onload = function() {
                        resolve(loadingImage);
                    };
                    loadingImage.onerror = function(e) {
                        reject(e);
                    };
                    loadingImage.src = uri;
                }).then(function(loadedImage) {
                    var imageData = obtainImageData(loadedImage);
                    return imageData;
                });
                return resultPromise;
            };

            /**
             * A function that returns a promise to the source code of the
             * shader with the given ID, as a string.
             *
             * @param {string} shaderId The shader ID
             * @return {Promise} The promise to the source code of the specified
             * shader
             */
            var shaderSourcePromiseFunction = function(shaderId) {
                var shader = gltf.shaders[shaderId];
                console.log("shader is " + shader + " for Id " + shaderId);
                var shaderUri = shader.uri;
                var uri = resolveUri(baseUri, shaderUri);
                var resultPromise = loadTextFile(uri);
                return resultPromise;
            };

            this.bufferDatas = new LazyMap(bufferDataPromiseFunction);
            this.imageDatas = new LazyMap(imageDataPromiseFunction);
            this.shaderSources = new LazyMap(shaderSourcePromiseFunction);

            /**
             * Returns a promise to the binary data (as an arraybuffer) of the
             * glTF buffer with the given ID
             *
             * @param {string} bufferId The glTF buffer ID
             * @return {Promise} The promise to the arraybuffer data
             */
            GltfData.prototype.fetchBufferDataPromise = function(bufferId) {
                return that.bufferDatas.getPromise(bufferId);
            };

            /**
             * Returns a promise to the source code (as a string) of the
             * glTF shader with the given ID
             *
             * @param {string} bufferId The glTF buffer ID
             * @return {Promise} The promise to the shader source code string
             */
            GltfData.prototype.fetchShaderSourcePromise = function(shaderId) {
                return that.shaderSources.getPromise(shaderId);
            };

            /**
             * Returns a promise to the ImageData of the glTF image with the
             * given ID
             *
             * @param {string} imageId The glTF image ID
             * @return {Promise} The promise to the ImageData of the glTF image
             */
            GltfData.prototype.fetchImageDataPromise = function(imageId) {
                return that.imageDatas.getPromise(imageId);
            };

        };

        gltfTutorial.GltfData = GltfData;

    })(gltfTutorial); 
    
});

