var gltfTutorial = gltfTutorial || {};

define([
    'Utils'
], function(
    Utils
) {
    (function(gltfTutorial) {
        'use strict';

        var isDefined = gltfTutorial.Utils.isDefined;

        /**
         * Constructor of a LazyMap.
         *
         * @param {Object} promiseFunction The function that computes the
         * promise to the value for a given key
         */
        var LazyMap = function(promiseFunction) {
            var that = this;

            this.promiseFunction = promiseFunction;
            this.values = {};
            this.promises = {};

            /**
             * Returns a promise for the value for the given key. The promise will
             * be fulfilled when the value is available. Repeated calls with the
             * same key will return the same promise, until the value is available.
             * When the value is available, the calls will return a fulfilled
             * promise to the value.
             *
             * @param {Object} key The key for the lookup
             * @return {Promise} The promise of the value for the given key
             */
            LazyMap.prototype.getPromise = function(key) {
                if (isDefined(that.values[key])) {
                    return Promise.resolve(that.values[key]);
                }
                if (isDefined(that.promises[key])) {
                    return that.promises[key];
                }
                var resultPromise = this.promiseFunction(key).then(function(value) {
                    that.values[key] = value;
                    delete that.promises[key];
                    return value;
                }, function(reason) {
                    delete that.promises[key];
                    reject(reason);
                });
                that.promises[key] = resultPromise;
                return resultPromise;
            };

            /**
             * Returns The value for the given key. This will be undefined until
             * the promise for the given key (that may be obtained with getPromise)
             * has been fulfilled.
             *
             * @param {Object} key The key for the lookup
             * @return {Object} The value for the given key
             */
            LazyMap.prototype.getValue = function(key) {
                return that.values[key];
            };

        };

        gltfTutorial.LazyMap = LazyMap;

    })(gltfTutorial);

});

