var gltfTutorial = gltfTutorial || {};

define([
    'Utils',
    'MatUtils',
    'GltfUtils',
    'external/gl-matrix'
], function (
    Utils,
    MatUtils,
    GltfUtils,
    glm
){


    (function(gltfTutorial) {
        'use strict';

        var isDefined = gltfTutorial.Utils.isDefined;
        var isUndefined = gltfTutorial.Utils.isUndefined;
        var optional = gltfTutorial.Utils.optional;
        var logMatrix = gltfTutorial.MatUtils.logMatrix;
        var getNumComponentsForAccessorType = gltfTutorial.GltfUtils.getNumComponentsForAccessorType;
        var LazyMap = gltfTutorial.LazyMap;


        /**
         * Constructor for a rendered glTF. It receives the GltfData and
         * allows rendering the glTF using the given GlContext. 
         * 
         * @param {GltfData} gltfData The GltfData that contains the actual
         * glTF, and has functions for obtaining the external data (buffers,
         * images and shaders) that is linked to from the glTF. 
         * @param {GlContext} glContext The GlContext that offers
         */
        var RenderedGltf = function(gltfData, glContext, renderCallback) {
            var that = this;

            this.gltfData = gltfData;
            this.gltf = gltfData.gltf;
            this.glContext = glContext;
            this.renderCallback = renderCallback;

            this.currentCameraNodeId = undefined;
            this.parentNodeIds = {};

            this.renderCommands = [];

            this.aspectRatioProvider = function() {
                var glContext = that.glContext;
                return glContext.glViewport.width / glContext.glViewport.height;
            };

            function glProgramPromiseFunction(programId) {
                var gltfData = that.gltfData;
                var gltf = gltfData.gltf;
                var program = gltf.programs[programId];
                var vertexShaderId = program.vertexShader;
                var vertexShaderSourcePromise = gltfData.fetchShaderSourcePromise(vertexShaderId);
                var fragmentShaderId = program.fragmentShader;
                var fragmentShaderSourcePromise = gltfData.fetchShaderSourcePromise(fragmentShaderId);
                var promises = [vertexShaderSourcePromise, fragmentShaderSourcePromise];
                var resultPromise = Promise.all(promises).then(function(shaderSources) {
                    var vertexShaderSource = shaderSources[0];
                    var fragmentShaderSource = shaderSources[1];
                    var glProgram = that.glContext.createGlProgram(vertexShaderSource, fragmentShaderSource);
                    return glProgram;
                });
                return resultPromise;
            }

            function glTexturePromiseFunction(textureId) {
                var gltf = that.gltf;
                var texture = gltf.textures[textureId];
                var imageId = texture.source;
                var image = gltf.images[imageId];
                var resultPromise = that.gltfData.fetchImageDataPromise(imageId).then(function(imageData) {
                    var glTexture = that.glContext.createGlTexture(texture, imageData);
                    return glTexture;
                });
                return resultPromise;
            }

            function glBufferViewPromiseFunction(bufferViewId) {
                var gltf = that.gltf;
                var bufferView = gltf.bufferViews[bufferViewId];
                var bufferId = bufferView.buffer;
                var resultPromise = that.gltfData.fetchBufferDataPromise(bufferId).then(function(bufferData) {
                    var glBufferView = that.glContext.createGlBuffer(bufferView, bufferData);
                    return glBufferView;
                });
                return resultPromise;
            }


            this.glPrograms = new LazyMap(glProgramPromiseFunction);
            this.glTextures = new LazyMap(glTexturePromiseFunction);
            this.glBufferViews = new LazyMap(glBufferViewPromiseFunction);

            RenderedGltf.prototype.init = function() {
                var gltf = that.gltf;
                var scenes = optional(gltf.scenes, {});
                Object.keys(scenes).forEach(function(sceneId, index) {
                    var scene = scenes[sceneId];
                    traverseNodes(scene.nodes);
                });
            };

            function traverseNodes(nodeIds) {
                if (isUndefined(nodeIds)) {
                    return;
                }
                var gltf = that.gltf;

                nodeIds.forEach(function(nodeId) {
                    var node = gltf.nodes[nodeId];
                    console.log("Traverse node with ID " + nodeId);

                    if (isDefined(node.camera)) {
                        // TODO Make currentCameraNodeId selectable!
                        that.currentCameraNodeId = nodeId;
                    }

                    processMeshes(node.meshes, nodeId);

                    var children = optional(node.children, []);
                    children.forEach(function(childId) {
                        that.parentNodeIds[childId] = nodeId;
                    });
                    traverseNodes(node.children);
                });
            };

            function processMeshes(meshIds, currentNodeId) {
                if (isUndefined(meshIds)) {
                    return;
                }
                var gltf = that.gltf;

                meshIds.forEach(function(meshId) {
                    var mesh = gltf.meshes[meshId];
                    console.log("Process mesh with ID " + meshId);
                    processMeshPrimitives(mesh.primitives, currentNodeId);
                });
            };

            function processMeshPrimitives(meshPrimitives, currentNodeId) {
                if (isUndefined(meshPrimitives)) {
                    return;
                }
                var gltf = that.gltf;

                meshPrimitives.forEach(function(meshPrimitive) {
                    console.log("Process meshPrimitive " + meshPrimitive);

                    var meshPrimitiveInitialization = createMeshPrimitiveInitialization(meshPrimitive);
                    meshPrimitiveInitialization.then(function() {

                        console.log("meshPrimitive initialization done, create render command");
                        createMeshPrimitiveRenderCommands(meshPrimitive, currentNodeId);

                        that.renderCallback();
                    });

                });
            };

            function createMeshPrimitiveInitialization(meshPrimitive) {

                var gltf = that.gltf;
                var glContext = that.glContext;
                var gl = glContext.gl;

                var materialId = meshPrimitive.material;
                var material = gltf.materials[materialId];
                var techniqueId = material.technique;
                var technique = gltf.techniques[techniqueId];
                var programId = technique.program;

                var promises = [];

                var glProgramPromise = that.glPrograms.getPromise(programId);
                promises.push(glProgramPromise);

                var techniqueAttributes = optional(technique.attributes, {});
                Object.keys(techniqueAttributes).forEach(function(attributeName, index) {

                    var techniqueParametersId = techniqueAttributes[attributeName];
                    var techniqueParameters = technique.parameters[techniqueParametersId];
                    var semantic = techniqueParameters.semantic;

                    var meshPrimitiveAttributes = meshPrimitive.attributes;
                    var accessorId = meshPrimitiveAttributes[semantic];
                    var accessor = gltf.accessors[accessorId];
                    var numComponents = getNumComponentsForAccessorType(accessor.type);
                    var bufferViewId = accessor.bufferView;
                    var bufferView = gltf.bufferViews[bufferViewId];
                    var bufferId = bufferView.buffer;
                    var glBufferViewPromise = that.glBufferViews.getPromise(bufferViewId);
                    promises.push(glBufferViewPromise);
                });

                if (isDefined(meshPrimitive.indices)) {
                    var accessor = gltf.accessors[meshPrimitive.indices];
                    var bufferViewId = accessor.bufferView;
                    var glBufferViewPromise = that.glBufferViews.getPromise(bufferViewId);
                    promises.push(glBufferViewPromise);
                }

                var uniforms = optional(technique.uniforms, {});
                Object.keys(uniforms).forEach(function(uniform, index) {
                    if (isSampler(uniform, technique)) {
                        var textureId = fetchUniformValue(uniform, material, technique);
                        var glTexturePromise = that.glTextures.getPromise(textureId);
                        promises.push(glTexturePromise);
                    }
                });

                var resultPromise = Promise.all(promises);
                return resultPromise;
            };

            function createMeshPrimitiveRenderCommands(meshPrimitive, currentNodeId) {
                var gltf = that.gltf;
                var glContext = that.glContext;
                var gl = glContext.gl;

                var materialId = meshPrimitive.material;
                var material = gltf.materials[materialId];
                var techniqueId = material.technique;
                var technique = gltf.techniques[techniqueId];
                var programId = technique.program;
                var glProgram = that.glPrograms.getValue(programId);

                var techniqueAttributes = optional(technique.attributes, {});
                var uniforms = optional(technique.uniforms, {});

                var useProgramCommand = function() {
                    gl.useProgram(glProgram);
                };
                that.renderCommands.push(useProgramCommand);

                if (isDefined(meshPrimitive.indices)) {

                    var accessor = gltf.accessors[meshPrimitive.indices];
                    var bufferViewId = accessor.bufferView;
                    var glBufferView = that.glBufferViews.getValue(bufferViewId);
                    var indicesBindingCommand = function() {
                        gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, glBufferView);
                    };
                    that.renderCommands.push(indicesBindingCommand);
                }

                var arbitraryAccessorCount = undefined;
                Object.keys(techniqueAttributes).forEach(function(attributeName, index) {

                    var techniqueParametersId = techniqueAttributes[attributeName];
                    var techniqueParameters = technique.parameters[techniqueParametersId];
                    var semantic = techniqueParameters.semantic;

                    var meshPrimitiveAttributes = meshPrimitive.attributes;
                    var accessorId = meshPrimitiveAttributes[semantic];
                    var accessor = gltf.accessors[accessorId];
                    var numComponents = getNumComponentsForAccessorType(accessor.type);
                    var bufferViewId = accessor.bufferView;
                    var glBufferView = that.glBufferViews.getValue(bufferViewId);

                    arbitraryAccessorCount = accessor.count;

                    var attributeLocation = gl.getAttribLocation(glProgram, attributeName);

                    var attributeEnablingCommand = function() {
                        gl.bindBuffer(gl.ARRAY_BUFFER, glBufferView);
                        gl.enableVertexAttribArray(attributeLocation);
                        gl.vertexAttribPointer(attributeLocation, numComponents, accessor.componentType, false, accessor.byteStride, accessor.byteOffset);
                    };
                    that.renderCommands.push(attributeEnablingCommand);
                });

                Object.keys(uniforms).forEach(function(uniform, index) {
                    var uniformSettingCommand = createUniformSettingCommand(uniform, material, technique, currentNodeId);
                    that.renderCommands.push(uniformSettingCommand);
                });

                if (isDefined(meshPrimitive.indices)) {
                    var indicesAccessor = gltf.accessors[meshPrimitive.indices];
                    var drawCommand = function() {
                        gl.drawElements(meshPrimitive.mode, indicesAccessor.count, indicesAccessor.componentType, 0);
                    };
                    that.renderCommands.push(drawCommand);
                } else {
                    var drawCommand = function() {
                        gl.drawArrays(meshPrimitive.mode, arbitraryAccessorCount);
                    };
                    that.renderCommands.push(drawCommand);
                };
            };

            function isSampler(uniform, technique) {
                var glContext = that.glContext;
                var gl = glContext.gl;

                var uniforms = technique.uniforms;
                var techniqueParametersId = uniforms[uniform];
                var parameters = technique.parameters;
                var techniqueParameters = parameters[techniqueParametersId];
                var semantic = techniqueParameters.semantic;
                var type = techniqueParameters.type;
                return type == gl.SAMPLER_2D;
            };

            function createUniformSettingCommand(uniform, material, technique, currentNodeId) {

                var gltf = that.gltf;
                var glContext = that.glContext;
                var gl = glContext.gl;

                var programId = technique.program;
                var glProgram = that.glPrograms.getValue(programId);
                gl.useProgram(glProgram);
                var uniformLocation = gl.getUniformLocation(glProgram, uniform);

                var uniformValueGetter = createUniformValueGetter(uniform, material, technique);
                var uniformValueSetter;

                if (isSampler(uniform, technique)) {
                    var textureCounter = 0;
                    /// TODO PROPERLY COUNT THIS UP!
                    uniformValueSetter = function(uniformLocation, uniformValue) {
                        var textureId = uniformValue;
                        var glTexture = that.glTextures.getValue(textureId);
                        gl.activeTexture(gl.TEXTURE0 + textureCounter);
                        gl.bindTexture(gl.TEXTURE_2D, glTexture);
                        gl.uniform1i(uniformLocation, textureCounter);
                    };
                } else {
                    uniformValueSetter = createUniformValueSetter(uniform, technique);
                }

                return function() {
                    var uniformValue = uniformValueGetter();
                    console.log("uniformValue for " + uniform + " is " + uniformValue);
                    uniformValueSetter(uniformLocation, uniformValue);
                };
            };

            function createUniformValueGetter(uniform, material, technique, currentNodeId) {
                var uniforms = technique.uniforms;
                var techniqueParametersId = uniforms[uniform];
                var parameters = technique.parameters;
                var techniqueParameters = parameters[techniqueParametersId];
                var semantic = techniqueParameters.semantic;
                if (isDefined(semantic)) {
                    return createSmanticBasedUniformValueGetter(uniform, material, technique, currentNodeId, semantic);
                } else {
                    return function() {
                        return fetchUniformValue(uniform, material, technique);
                    };
                }
            };

            function createSmanticBasedUniformValueGetter(uniform, material, technique, currentNodeId, semantic) {

                var uniforms = technique.uniforms;
                var techniqueParametersId = uniforms[uniform];
                var parameters = technique.parameters;
                var techniqueParameters = parameters[techniqueParametersId];
                var accessedNodeId;
                if (isDefined(techniqueParameters.node)) {
                    accessedNodeId = techniqueParameters.node;
                } else {
                    accessedNodeId = currentNodeId;
                }

                switch (semantic) {
                case "LOCAL": {
                    return function() {
                        return computeLocalMatrix(accessedNodeId);
                    };
                }

                case "MODEL": {
                    return function() {
                        return computeGlobalMatrix(accessedNodeId);
                    };
                }

                case "VIEW": {
                    return function() {
                        return computeViewMatrix();
                    };
                }

                case "PROJECTION": {
                    return function() {
                        return computeProjectionMatrix();
                    };
                }

                case "MODELVIEW": {
                    return function() {
                        var viewMatrix = computeViewMatrix();
                        var modelMatrix = computeGlobalMatrix(accessedNodeId);
                        var result = glm.mat4.create();
                        glm.mat4.mul(result, result, viewMatrix);
                        glm.mat4.mul(result, result, modelMatrix);
                        return result;
                    };
                }

                case "MODELVIEWPROJECTION": {
                    return function() {
                        var projectionMatrix = computeProjectionMatrix();
                        var viewMatrix = computeViewMatrix();
                        var modelMatrix = computeGlobalMatrix(accessedNodeId);
                        var result = glm.mat4.create();
                        glm.mat4.mul(result, result, projectionMatrix);
                        glm.mat4.mul(result, result, viewMatrix);
                        glm.mat4.mul(result, result, modelMatrix);
                        return result;
                    };
                }

                case "MODELINVERSE": {
                    return function() {
                        var result = computeGlobalMatrix(accessedNodeId);
                        glm.mat4.invert(result, result);
                        return result;
                    };
                }

                case "VIEWINVERSE": {
                    return function() {
                        var result = computeViewMatrix();
                        glm.mat4.invert(result, result);
                        return result;
                    };
                }

                case "MODELVIEWINVERSE": {
                    return function() {
                        var viewMatrix = computeViewMatrix();
                        var modelMatrix = computeGlobalMatrix(accessedNodeId);
                        var result = glm.mat4.create();
                        glm.mat4.mul(result, result, viewMatrix);
                        glm.mat4.mul(result, result, modelMatrix);
                        glm.mat4.invert(result, result);
                        return result;
                    };
                }

                case "PROJECTIONINVERSE": {
                    return function() {
                        var result = computeProjectionMatrix();
                        glm.mat4.invert(result, result);
                        return result;
                    };
                }

                case "MODELVIEWPROJECTIONINVERSE": {
                    return function() {
                        var projectionMatrix = computeProjectionMatrix();
                        var viewMatrix = computeViewMatrix();
                        var modelMatrix = computeGlobalMatrix(accessedNodeId);
                        var result = glm.mat4.create();
                        glm.mat4.mul(result, result, projectionMatrix);
                        glm.mat4.mul(result, result, viewMatrix);
                        glm.mat4.mul(result, result, modelMatrix);
                        glm.mat4.invert(result, result);
                        return result;
                    };
                }

                case "MODELINVERSETRANSPOSE": {
                    return function() {
                        var matrix = computeGlobalMatrix(accessedNodeId);
                        glm.mat4.invert(matrix, matrix);
                        glm.mat4.transpose(matrix, matrix);
                        var result = glm.mat3.create();
                        glm.mat3.fromMat4(result, matrix);
                        return result;
                    };
                }

                case "MODELVIEWINVERSETRANSPOSE": {
                    return function() {
                        var viewMatrix = computeViewMatrix();
                        var modelMatrix = computeGlobalMatrix(accessedNodeId);
                        var matrix = glm.mat4.create();
                        glm.mat4.mul(matrix, matrix, viewMatrix);
                        glm.mat4.mul(matrix, matrix, modelMatrix);
                        glm.mat4.invert(matrix, matrix);
                        glm.mat4.transpose(matrix, matrix);
                        var result = glm.mat3.create();
                        glm.mat3.fromMat4(result, matrix);
                        return result;
                    };
                }

                case "VIEWPORT": {
                    return function() {
                        var glContext = that.glContext;
                        var glViewport = glContext.glViewport;
                        return [0, 0, glViewport.width, glViewport.height];
                    };
                }

                case "JOINTMATRIX": {
                    // TODO JOINTMATRIX is not supported yet
                    console.log("JOINTMATRIX is not supported yet");
                    break;
                }
                default:
                    break;
                }
                return function() {
                    console.log("invalid semantic: " + semantic);
                };
            };

            /**
             * Fetch the value of the specified uniform from the given material,
             * if it is defined there. If it is not defined in the material, then
             * return the default value from the technique.
             *
             * @param {String} uniform The uniform name
             * @param {Object} material The glTF material
             * @param {Object} technique The glTF technique
             * @return {Object} The value of the uniform
             */
            function fetchUniformValue(uniform, material, technique) {

                var uniforms = technique.uniforms;
                var techniqueParametersId = uniforms[uniform];
                var materialValues = material.values;
                if (isDefined(materialValues[techniqueParametersId])) {
                    return materialValues[techniqueParametersId];
                }
                var parameters = technique.parameters;
                var techniqueParameters = parameters[techniqueParametersId];
                return techniqueParameters.value;
            };

            function computeLocalMatrix(nodeId) {
                var gltf = that.gltf;
                var node = gltf.nodes[nodeId];

                var localMatrix = glm.mat4.create();
                if (isDefined(node.matrix)) {
                    glm.mat4.copy(localMatrix, node.matrix);
                    return localMatrix;
                }

                if (isDefined(node.rotation)) {
                    var translation = optional(node.translation, [0, 0, 0]);
                    if (isDefined(node.scale)) {
                        glm.mat4.fromRotationTranslationScale(localMatrix, node.rotation, translation, node.scale);
                    } else {
                        glm.mat4.fromRotationTranslation(localMatrix, node.rotation, translation);
                    }
                } else {
                    if (isDefined(node.translation)) {
                        glm.mat4.translate(localMatrix, node.translation);
                    }
                    if (isDefined(node.scale)) {
                        glm.mat4.translate(localMatrix, node.scale);
                    }
                }
                return localMatrix;
            };

            function computeGlobalMatrix(nodeId) {
                if (isUndefined(nodeId)) {
                    return glm.mat4.create();
                }
                var parentNodeId = that.parentNodeIds[nodeId];
                if (isUndefined(parentNodeId)) {
                    return computeLocalMatrix(nodeId);
                }
                var parentGlobalMatrix = computeGlobalMatrix(parentNodeId);
                var localMatrix = computeLocalMatrix(nodeId);
                return parentGlobalMatrix * localMatrix;
            };

            function computeProjectionMatrixOrthographic(projectionMatrix, cameraOrthographic) {
                glm.mat4.identity(projectionMatrix);
                var left = -cameraOrthographic.xmag * 0.5;
                var right = cameraOrthographic.xmag * 0.5;
                var botttom = -cameraOrthographic.ymag * 0.5;
                var top = cameraOrthographic.ymag * 0.5;
                glm.mat4.ortho(projectionMatrix, left, right, bottom, top, cameraOrthographic.znear, cameraOrthographic.zfar);
            };

            function computeProjectionMatrixPerspective(projectionMatrix, cameraPerspective) {
                glm.mat4.identity(projectionMatrix);

                var aspectRatio = cameraPerspective.aspectRatio;
                if (isDefined(that.aspectRatioProvider)) {
                    aspectRatio = that.aspectRatioProvider();
                }

                glm.mat4.perspective(projectionMatrix, cameraPerspective.yfov, aspectRatio, cameraPerspective.znear, cameraPerspective.zfar);
            };

            function computeProjectionMatrix() {
                var gltf = that.gltf;

                var projectionMatrix = glm.mat4.create();
                if (isDefined(that.currentCameraNodeId)) {
                    var cameraNode = gltf.nodes[that.currentCameraNodeId];
                    var camera = gltf.cameras[cameraNode.camera];
                    if (camera.type === "orthographic") {
                        computeProjectionMatrixOrthographic(projectionMatrix, camera.orthographic);
                    } else if (camera.type === "perspective") {
                        computeProjectionMatrixPerspective(projectionMatrix, camera.perspective);
                    } else {
                        console.log("Invalid camera type: " + camera.type);
                    }

                }
                
                console.log("Projection matrix:");
                logMatrix(projectionMatrix);
                
                return projectionMatrix;
            };

            function computeViewMatrix() {
                if (isUndefined(that.currentCameraNodeId)) {
                    return glm.mat4.create();
                }
                var matrix = computeGlobalMatrix(that.currentCameraNodeId);
                glm.mat4.invert(matrix, matrix);
                return matrix;
            };

            function createSetter(gl, glFunction) {
                return function(location, value) {
                    glFunction.call(gl, location, value);
                };
            }

            function createMatrixSetter(gl, glFunction) {
                return function(location, value) {
                    glFunction.call(gl, location, false, value);
                };
            }

            function createUniformValueSetter(uniform, technique) {

                var glContext = that.glContext;
                var gl = glContext.gl;

                var uniforms = technique.uniforms;
                var techniqueParametersId = uniforms[uniform];
                var parameters = technique.parameters;
                var techniqueParameters = parameters[techniqueParametersId];
                var semantic = techniqueParameters.semantic;
                var type = techniqueParameters.type;
                var count = optional(techniqueParameters.count, 1);

                switch (type) {
                case gl.INT: {
                    if (count === 1) {
                        return createSetter(gl, gl.uniform1i);
                    }
                    return createSetter(gl, gl.uniform1iv);
                }
                case gl.UNSIGNED_INT: {
                    if (count === 1) {
                        return createSetter(gl, gl.uniform1ui);
                    }
                    return createSetter(gl, gl.uniform1uiv);
                }
                case gl.INT_VEC2:
                    return createSetter(gl, gl.uniform2iv);
                case gl.INT_VEC3:
                    return createSetter(gl, gl.uniform3iv);
                case gl.INT_VEC4:
                    return createSetter(gl, gl.uniform4iv);
                case gl.FLOAT: {
                    if (count === 1) {
                        return createSetter(gl, gl.uniform1f);
                    }
                    return createSetter(gl, gl.uniform1fv);
                }
                case gl.FLOAT_VEC2:
                    return createSetter(gl, gl.uniform2fv);
                case gl.FLOAT_VEC3:
                    return createSetter(gl, gl.uniform3fv);
                case gl.FLOAT_VEC4:
                    return createSetter(gl, gl.uniform4fv);
                case gl.FLOAT_MAT2:
                    return createMatrixSetter(gl, gl.uniformMatrix2fv);
                case gl.FLOAT_MAT3:
                    return createMatrixSetter(gl, gl.uniformMatrix3fv);
                case gl.FLOAT_MAT4:
                    return createMatrixSetter(gl, gl.uniformMatrix4fv);
                case gl.SAMPLER_2D:
                    break;
                default:
                    break;
                }
                return function() {
                    console.log("Invalid type for uniform " + uniform + ": " + type);
                };
            };

            RenderedGltf.prototype.render = function() {
                that.renderCommands.forEach(function(renderCommand) {
                    renderCommand();
                });
            };

            function computeLocalMatrix_Alternative(nodeId) {
                var gltf = that.gltf;
                var node = gltf.nodes[nodeId];

                var localMatrix = glm.mat4.create();
                if (isDefined(node.matrix)) {
                    glm.mat4.copy(localMatrix, node.matrix);
                    return localMatrix;
                }

                if (isDefined(node.translation)) {
                    var translationMatrix = glm.mat4.create();
                    glm.mat4.fromTranslation(translationMatrix, node.translation);
                    glm.mat4.mul(localMatrix, localMatrix, translationMatrix);
                }
                if (isDefined(node.rotation)) {
                    var quaternion = quat.create();
                    quat.copy(quaternion, node.rotation);
                    var rotationMatrix = glm.mat4.create();
                    glm.mat4.fromQuat(rotationMatrix, quaternion);
                    glm.mat4.mul(localMatrix, localMatrix, rotationMatrix);
                }
                if (isDefined(node.scale)) {
                    var scaleMatrix = glm.mat4.create();
                    glm.mat4.fromScale(scaleMatrix, node.scale);
                    glm.mat4.mul(localMatrix, localMatrix, translationMatrix);
                }
                return localMatrix;
            };

        };

        gltfTutorial.RenderedGltf = RenderedGltf;

    })(gltfTutorial); 
    
});
